@@ -1,16 +1,17 @@
 package Crypt::PBKDF2; 
 # ABSTRACT: The PBKDF2 password hashing algorithm.
-our $VERSION = '0.131750'; # VERSION
+our $VERSION = '0.142390'; # VERSION
 our $AUTHORITY = 'cpan:ARODLAND'; # AUTHORITY
 use Moose 1;
-use Method::Signatures::Simple;
 use Moose::Util::TypeConstraints;
 use namespace::autoclean;
 use MIME::Base64 ();
 use Carp qw(croak);
+use Module::Runtime;
 use Try::Tiny;
 
-method BUILD {
+sub BUILD {
+  my ($self) = @_;
   $self->hasher; # Force instantiation, so we get errors ASAP
 }
 
@@ -47,15 +48,15 @@ has _lazy_hasher => (
   builder => '_build_hasher',
 );
 
-method _build_hasher {
+sub _build_hasher {
+  my ($self) = @_;
   my $class = $self->hash_class;
   if ($class !~ s/^\+//) {
     $class = "Crypt::PBKDF2::Hash::$class";
   }
   my $hash_args = $self->hash_args;
 
-  Class::MOP::load_class($class);
-  return $class->new( %$hash_args );
+  return Module::Runtime::use_module($class)->new( %$hash_args );
 }
 
 
@@ -79,7 +80,8 @@ has salt_len => (
   default => 4,
 );
 
-method _random_salt {
+sub _random_salt {
+  my ($self) = @_;
   my $ret = "";
   for my $n (1 .. $self->salt_len) {
     $ret .= chr(int rand 256);
@@ -95,15 +97,33 @@ has encoding => (
 );
 
 
-method generate ($password, $salt) {
+has length_limit => (
+  is => 'ro',
+  isa => 'Int',
+  predicate => 'has_length_limit',
+);
+
+
+sub generate {
+  my ($self, $password, $salt) = @_;
   $salt = $self->_random_salt unless defined $salt;
 
+  if ($self->has_length_limit and length($password) > $self->length_limit) {
+    croak "Password exceeds length limit";
+  }
+
   my $hash = $self->PBKDF2($salt, $password);
   return $self->encode_string($salt, $hash);
 }
 
 
-method validate ($hashed, $password) {
+sub validate {
+  my ($self, $hashed, $password) = @_;
+
+  if ($self->has_length_limit and length($password) > $self->length_limit) {
+    croak "Password exceeds length limit";
+  }
+
   my $info = $self->decode_string($hashed);
 
   my $hasher = try {
@@ -125,7 +145,8 @@ method validate ($hashed, $password) {
 }
 
 
-method PBKDF2 ($salt, $password) {
+sub PBKDF2 {
+  my ($self, $salt, $password) = @_;
   my $iterations = $self->iterations;
   my $hasher = $self->hasher;
   my $output_len = $self->output_len || $hasher->hash_len;
@@ -164,7 +185,8 @@ sub PBKDF2_hex {
   return unpack "H*", unpack "A*", $self->PBKDF2(@_);
 }
 
-method _PBKDF2_F ($hasher, $salt, $password, $iterations, $i) {
+sub _PBKDF2_F {
+  my ($self, $hasher, $salt, $password, $iterations, $i) = @_;
   my $result = 
   my $hash = 
     $hasher->generate( $salt . pack("N", $i), $password );
@@ -178,7 +200,8 @@ method _PBKDF2_F ($hasher, $salt, $password, $iterations, $i) {
 }
 
 
-method encode_string ($salt, $hash) {
+sub encode_string {
+  my ($self, $salt, $hash) = @_;
   if ($self->encoding eq 'crypt') {
     return $self->_encode_string_cryptlike($salt, $hash);
   } elsif ($self->encoding eq 'ldap') {
@@ -188,7 +211,8 @@ method encode_string ($salt, $hash) {
   }
 }
 
-method _encode_string_cryptlike ($salt, $hash) {
+sub _encode_string_cryptlike {
+  my ($self, $salt, $hash) = @_;
   my $hasher = $self->hasher;
   my $hasher_class = Class::MOP::class_of($hasher)->name;
   if (!defined $hasher_class || $hasher_class !~ s/^Crypt::PBKDF2::Hash:://) {
@@ -203,7 +227,8 @@ method _encode_string_cryptlike ($salt, $hash) {
   . MIME::Base64::encode($hash, "");
 }
 
-method _encode_string_ldaplike ($salt, $hash) {
+sub _encode_string_ldaplike {
+  my ($self, $salt, $hash) = @_;
   my $hasher = $self->hasher;
   my $hasher_class = Class::MOP::class_of($hasher)->name;
   if (!defined $hasher_class || $hasher_class !~ s/^Crypt::PBKDF2::Hash:://) {
@@ -220,7 +245,8 @@ method _encode_string_ldaplike ($salt, $hash) {
 }
 
 
-method decode_string ($hashed) {
+sub decode_string {
+  my ($self, $hashed) = @_;
   if ($hashed =~ /^\$PBKDF2\$/) {
     return $self->_decode_string_cryptlike($hashed);
   } elsif ($hashed =~ /^\{X-PBKDF2}/i) {
@@ -230,7 +256,8 @@ method decode_string ($hashed) {
   }
 }
 
-method _decode_string_cryptlike ($hashed) {
+sub _decode_string_cryptlike {
+  my ($self, $hashed) = @_;
   if ($hashed !~ /^\$PBKDF2\$/) {
     croak "Unrecognized hash";
   }
@@ -249,7 +276,8 @@ method _decode_string_cryptlike ($hashed) {
   }
 }
 
-method _decode_string_ldaplike ($hashed) {
+sub _decode_string_ldaplike {
+  my ($self, $hashed) = @_;
   if ($hashed !~ /^\{X-PBKDF2}/i) {
     croak "Unrecognized hash";
   }
@@ -270,18 +298,20 @@ method _decode_string_ldaplike ($hashed) {
 }
 
 
-method hasher_from_algorithm ($algorithm, $args) {
+sub hasher_from_algorithm {
+  my ($self, $algorithm, $args) = @_;
+  my $class = Module::Runtime::use_module("Crypt::PBKDF2::Hash::$algorithm");
+
   if (defined $args) {
-    Class::MOP::load_class( "Crypt::PBKDF2::Hash::$algorithm" );
-    return "Crypt::PBKDF2::Hash::$algorithm"->from_algo_string($args);
+    return $class->from_algo_string($args);
   } else {
-    Class::MOP::load_class( "Crypt::PBKDF2::Hash::$algorithm" );
-    return "Crypt::PBKDF2::Hash::$algorithm"->new;
+    return $class->new;
   }
 }
 
 
-method clone (%params) {
+sub clone {
+  my ($self, %params) = @_;
   my $class = ref $self;
 
   # If the hasher was built from hash_class and hash_args, then omit it from
@@ -303,13 +333,15 @@ method clone (%params) {
   return $class->new(%new_args);
 }
 
-method _b64_encode_int32 ($value) {
+sub _b64_encode_int32 {
+  my ($self, $value) = @_;
   my $b64 = MIME::Base64::encode(pack("N", $value), "");
   $b64 =~ s/==$//;
   return $b64;
 }
 
-method _b64_decode_int32 ($b64) {
+sub _b64_decode_int32 {
+  my ($self, $b64) = @_;
   $b64 .= "==";
   return unpack "N", MIME::Base64::decode($b64);
 }
@@ -321,13 +353,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Crypt::PBKDF2 - The PBKDF2 password hashing algorithm.
 
 =head1 VERSION
 
-version 0.131750
+version 0.142390
 
 =head1 SYNOPSIS
 
@@ -421,6 +455,19 @@ Versions of this module up to 0.110461 generated the "crypt" format, so set
 that if you want it. Current versions of this module will read either format,
 but the "ldap" format is preferred.
 
+=head2 length_limit
+
+B<Type:> Integer
+
+The maximum password length to allow, for generate and verify functions.
+Allowing passwords of unlimited length can allow a denial-of-service attack
+in which an attacker asks the server to validate very large passwords.
+
+For compatibility this attribute is unset by default, but it is recommended
+to set it to a reasonably small value like 100 -- large enough that users
+aren't discouraged from having secure passwords, but small enough to limit
+the computation needed to validate any one password.
+
 =head1 METHODS
 
 =head2 generate ($password, [$salt])
@@ -522,7 +569,7 @@ Andrew Rodland <arodland@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Andrew Rodland.
+This software is copyright (c) 2014 by Andrew Rodland.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
